"use strict";

this.name        = "Sniper Sight";
this.author      = "Wildeblood";
this.copyright   = "(C)2011, Wildeblood"; // All rights reserved. Free to use, but DO NOT PROMULGATE MODIFIED VERSIONS.
this.description = "Wildefire Systems Type 1D Gunsight";
this.version     = "2.0.3";

/* ====================================================================================
			VARIOUS VARIABLES

	Variables that are _Capitalized can be configured by the user,
	variables that are _camelCase are used internally.
======================================================================================= */

this.startUp = function()
	{
	// Select the sniper HUD to use, depending on contrast setting.
	if (missionVariables.sniperScope_Contrast === null) this.sniperScope_Contrast = 3;
	else this.sniperScope_Contrast = missionVariables.sniperScope_Contrast;
	this.sniper_hud = "sniper-hud-contrast-" + this.sniperScope_Contrast + ".plist";

	this.sniperScope_Camera      = missionVariables.sniperScope_Camera;
	this.sniperScope_menuState   = missionVariables.sniperScope_menuState;
	this.sniperScope_Mode        = missionVariables.sniperScope_Mode;
	this.sniperScope_Sensitivity = missionVariables.sniperScope_Sensitivity;

	this.sniperScope_currentSight = ("EQ_TARGET_SCOPE_" + missionVariables.sniperScope_Size + "_" + missionVariables.sniperScope_Opacity);
	this.sniperScope_currentGlow  = ("EQ_TARGET_SCOPE_" + missionVariables.sniperScope_Size + "_" + missionVariables.sniperScope_Glow);
	this.sniperScope_currentHold  = ("EQ_TARGET_SCOPE_HOLD_" + missionVariables.sniperScope_CentreColour);
	this.sniperScope_currentState = "OFF_TARGET";
	}

this.playerWillSaveGame = function()
	{
	if (player.ship.equipmentStatus("EQ_SNIPER_SCOPE") === "EQUIPMENT_OK")
		{
		missionVariables.sniperScope_Camera      = this.sniperScope_Camera;
		missionVariables.sniperScope_Contrast    = this.sniperScope_Contrast;
		missionVariables.sniperScope_menuState   = this.sniperScope_menuState;
		missionVariables.sniperScope_Mode        = this.sniperScope_Mode;
		missionVariables.sniperScope_Sensitivity = this.sniperScope_Sensitivity;
		}
	}

/* ====================================================================================
			SWITCHING TO SNIPER HUD
======================================================================================= */

this.compassTargetChanged = function(whom, mode)
	{
	if (mode === "COMPASS_MODE_TARGET" && player.ship.hud !== this.sniper_hud && player.ship.equipmentStatus("EQ_SNIPER_SCOPE") === "EQUIPMENT_OK")
		{
		this.switchback_hud = player.ship.hud;
		player.ship.hud = this.sniper_hud;
		this.ssta = addFrameCallback(this.sniperScope_targetAlert.bind(this));

		player.consoleMessage("Gunsight target: " + whom.displayName + ".",3);
		if (this.sniperScope_menuState !== "MENU_SHOWN")
			{
			player.ship.awardEquipment("EQ_TARGET_SCOPE_CONFIG_MENU_ON");
			return;
			}
		if (this.sniperScope_menuState === "MENU_SHOWN")
			{
			this.sniperScope_showMenu();
			return;
			}
		}
	if (mode !== "COMPASS_MODE_TARGET")
		{
		this.sniperScope_switchback();
		}
	}

/* ====================================================================================
			WHEN ELSE TO SWITCH OFF SNIPER HUD
======================================================================================= */

this.shipWillDockWithStation = this.shipWillEnterWitchspace = this.shipDied = function()
	{
	this.sniperScope_switchback();
	}

this.guiScreenChanged = function()
	{
	if (guiScreen !== "GUI_SCREEN_MAIN")
		{
		this.sniperScope_switchback();
		}
	}

/* ====================================================================================
			SWITCHING BACK TO NORMAL HUD
======================================================================================= */

this.sniperScope_switchback = function()
	{
	if (player.ship.hud === this.sniper_hud)
		{
		player.ship.hud = this.switchback_hud;
		delete this.switchback_hud;
		if (this.ssta)
			{
			removeFrameCallback(this.ssta);
			delete this.ssta;
			if (this.sniperScope_currentState === "ON_HOLD") player.ship.removeEquipment(this.sniperScope_currentHold);
			player.ship.removeEquipment(this.sniperScope_currentGlow);
			player.ship.awardEquipment(this.sniperScope_currentSight);
			this.sniperScope_currentState = "OFF_TARGET";
			}
		if (missionVariables.reticleTargetSensitive === "TRUE")
			{
			this.reticleTargetSensitiveTimer = new Timer(this, sniperScope_reticleTargetSensitive, 0.25);
			}
		if (this.sniperScope_menuState === "MENU_HIDDEN")
			{
			player.ship.removeEquipment("EQ_TARGET_SCOPE_CONFIG_MENU_ON");
			return;
			}
		if (this.sniperScope_menuState === "MENU_SHOWN")
			{
			this.sniperScope_hideMenu();
			return;
			}
		}
	}

// Some bogosity to play nicely with Reticle Target Sensitive OXP:

this.sniperScope_reticleTargetSensitive = function()
	{
	player.ship.reticleTargetSensitive = true;
	}

/* ====================================================================================
			TARGET SENSITIVITY & HOLD MODE (FRAME CALLBACK)
======================================================================================= */

this.sniperScope_targetAlert = function()
	{
	if (player.ship.target)
		{
		if (this.sniperScope_currentState === "ON_HOLD") player.ship.removeEquipment(this.sniperScope_currentHold);

		var self = player.ship;
		var target = player.ship.target;
		var range = target.position.distanceTo(self.position) - target.collisionRadius;
		var deviation; // Deviation in metres.
		switch (self.viewDirection)
			{
			case "VIEW_PORT":      deviation = self.vectorRight.angleTo(self.position.subtract(target.position)) * range;   break;
			case "VIEW_STARBOARD": deviation = self.vectorRight.angleTo(target.position.subtract(self.position)) * range;   break;
			case "VIEW_AFT":       deviation = self.vectorForward.angleTo(self.position.subtract(target.position)) * range; break;
			case "VIEW_FORWARD":
			default:               deviation = self.vectorForward.angleTo(target.position.subtract(self.position)) * range;
			}
		if (deviation < (target.collisionRadius / this.sniperScope_Sensitivity))
			{
			if (this.sniperScope_currentState === "ON_TARGET") return;

			player.ship.removeEquipment(this.sniperScope_currentSight);
			player.ship.awardEquipment(this.sniperScope_currentGlow);
			this.sniperScope_currentState = "ON_TARGET";
			// player.consoleMessage("SHOOT",1);
			return;
			}
		else
			{
			if (this.sniperScope_currentState === "OFF_TARGET") return;

			player.ship.removeEquipment(this.sniperScope_currentGlow);
			player.ship.awardEquipment(this.sniperScope_currentSight);
			this.sniperScope_currentState = "OFF_TARGET";
			return;
			}
		}
	// Only reaches here if target has gone.
	if (this.sniperScope_Mode === "HOLD") // Voodoo here.
		{
		if (this.sniperScope_currentState === "ON_HOLD") return;

		player.ship.removeEquipment(this.sniperScope_currentGlow);
		player.ship.removeEquipment(this.sniperScope_currentSight);
		player.ship.awardEquipment(this.sniperScope_currentHold);
		this.sniperScope_currentState = "ON_HOLD";
		return;
		}
	if (this.sniperScope_Mode === "RELEASE")
		{
		if (this.sniperScope_currentState === "OFF_TARGET") return;

		player.ship.removeEquipment(this.sniperScope_currentGlow);
		player.ship.awardEquipment(this.sniperScope_currentSight);
		this.sniperScope_currentState = "OFF_TARGET";
		return;
		}
	}

/* ====================================================================================
			THE CAMERA
======================================================================================= */

this.shipKilledOther = function(whom)
	{
	if (player.ship.hud === this.sniper_hud && this.sniperScope_Camera === "ON")
		{
		log(this.name,"killed other ship " + whom + ".");
		takeSnapShot("Sniper-Sight");
		}
	}

/* ====================================================================================
			SET-UP & CLEAN-UP WHEN EQUIPMENT IS BOUGHT & SOLD
======================================================================================= */

this.playerBoughtEquipment = function(equipmentKey)
	{
	if (equipmentKey === "EQ_SNIPER_SCOPE")
		{
		// Avoid over-writing user's settings when repairing via F3 screen.
		if (missionVariables.sniperScope_damaged === null)
			{
			player.ship.awardEquipment("EQ_TARGET_SCOPE_4_4");
			player.ship.awardEquipment("EQ_TARGET_SCOPE_RETICLE");
			this.sniperScope_Camera = "OFF";
			this.sniperScope_Mode = "RELEASE";
			this.sniperScope_Sensitivity = 3;
			this.sniperScope_currentSight = "EQ_TARGET_SCOPE_4_4";
			this.sniperScope_currentGlow = "EQ_TARGET_SCOPE_4_9";
			this.sniperScope_currentHold = "EQ_TARGET_SCOPE_HOLD_G";
			this.sniperScope_menuState = "MENU_HIDDEN";
			missionVariables.sniperScope_Centre = 0;
			missionVariables.sniperScope_CentreColour = "G";
			missionVariables.sniperScope_nextCentre = 1;
			missionVariables.sniperScope_Contrast = 3;
			missionVariables.sniperScope_nextContrast = 1;
			missionVariables.sniperScope_Glow = 9;
			missionVariables.sniperScope_Opacity = 4;
			missionVariables.sniperScope_Size = 4;
			}
		else delete missionVariables.sniperScope_damaged;
		}
	if (equipmentKey === "EQ_SNIPER_SCOPE_REMOVAL")
		{
		player.ship.removeEquipment(this.sniperScope_currentSight);
		if ((missionVariables.sniperScope_Centre < 6) && (missionVariables.sniperScope_Centre > 0))
			{
			player.ship.removeEquipment("EQ_TARGET_SCOPE_PIP_" + missionVariables.sniperScope_CentreColour + missionVariables.sniperScope_Centre);
			}
		if (player.ship.equipmentStatus("EQ_TARGET_SCOPE_RETICLE") === "EQUIPMENT_OK")
			{
			player.ship.removeEquipment("EQ_TARGET_SCOPE_RETICLE");
			}
		player.ship.removeEquipment("EQ_SNIPER_SCOPE");
		player.ship.removeEquipment("EQ_SNIPER_SCOPE_REMOVAL");

		player.credits += (EquipmentInfo.infoForKey("EQ_SNIPER_SCOPE").price * 0.1 * 0.55);

		delete missionVariables.sniperScope_Camera;
		delete missionVariables.sniperScope_Centre;
		delete missionVariables.sniperScope_CentreColour;
		delete missionVariables.sniperScope_Contrast;
		delete missionVariables.sniperScope_Glow;
		delete missionVariables.sniperScope_Mode;
		delete missionVariables.sniperScope_Opacity;
		delete missionVariables.sniperScope_Sensitivity;
		delete missionVariables.sniperScope_Size;
		delete missionVariables.sniperScope_menuState;
		delete missionVariables.sniperScope_nextCentre;
		delete missionVariables.sniperScope_nextContrast;
		}
	}

/* ====================================================================================
			SHOWING & HIDING THE CONFIGURATION MENU
======================================================================================= */

this.sniperScope_showMenu = function()
	{
	// The order equipment is awarded is the order it will appear in on the N-menu.
	player.ship.awardEquipment("EQ_TARGET_SCOPE_CONFIG_SIZE");
	player.ship.awardEquipment("EQ_TARGET_SCOPE_CONFIG_MODE");
	player.ship.awardEquipment("EQ_TARGET_SCOPE_CONFIG_CENTRE");
	player.ship.awardEquipment("EQ_TARGET_SCOPE_CONFIG_OPACITY");
	player.ship.awardEquipment("EQ_TARGET_SCOPE_CONFIG_CONTRAST");
	player.ship.awardEquipment("EQ_TARGET_SCOPE_CONFIG_RETICLE");
	player.ship.awardEquipment("EQ_TARGET_SCOPE_CONFIG_CAMERA");
	player.ship.awardEquipment("EQ_TARGET_SCOPE_CONFIG_CENTRE_COLOUR");
	player.ship.awardEquipment("EQ_TARGET_SCOPE_CONFIG_GLOW");
	player.ship.awardEquipment("EQ_TARGET_SCOPE_CONFIG_SENSITIVITY");
	player.ship.awardEquipment("EQ_TARGET_SCOPE_CONFIG_MENU_OFF");
	}

this.sniperScope_hideMenu = function()
	{
	player.ship.removeEquipment("EQ_TARGET_SCOPE_CONFIG_CAMERA");
	player.ship.removeEquipment("EQ_TARGET_SCOPE_CONFIG_CENTRE");
	player.ship.removeEquipment("EQ_TARGET_SCOPE_CONFIG_CENTRE_COLOUR");
	player.ship.removeEquipment("EQ_TARGET_SCOPE_CONFIG_CONTRAST");
	player.ship.removeEquipment("EQ_TARGET_SCOPE_CONFIG_GLOW");
	player.ship.removeEquipment("EQ_TARGET_SCOPE_CONFIG_MODE");
	player.ship.removeEquipment("EQ_TARGET_SCOPE_CONFIG_OPACITY");
	player.ship.removeEquipment("EQ_TARGET_SCOPE_CONFIG_RETICLE");
	player.ship.removeEquipment("EQ_TARGET_SCOPE_CONFIG_SENSITIVITY");
	player.ship.removeEquipment("EQ_TARGET_SCOPE_CONFIG_SIZE");
	player.ship.removeEquipment("EQ_TARGET_SCOPE_CONFIG_MENU_OFF");
	}

/* ====================================================================================
			DAMAGE CONTROL

	Target scopes get fixed magically because they are hidden from the
	player, and the HUD will jam without one. Menu items get fixed magically
	because they are only menu items. Other items - centre pips and
	floating reticle - are not fixed by this script, but the player can
	repair them in-game simply by cycling the related configuration menu.
======================================================================================= */

this.equipmentDamaged = this.equipmentDestroyed = function(equipment)
	{
	switch(equipment)
		{
		case "EQ_SNIPER_SCOPE":
			{
			missionVariables.sniperScope_damaged = "DAMAGED";
			if (this.version < 2) log(this.name, equipment + " damaged.");
			return;
			}
		case "EQ_TARGET_SCOPE_0_3":
		case "EQ_TARGET_SCOPE_0_4":
		case "EQ_TARGET_SCOPE_0_5":
		case "EQ_TARGET_SCOPE_1_3":
		case "EQ_TARGET_SCOPE_1_4":
		case "EQ_TARGET_SCOPE_1_5":
		case "EQ_TARGET_SCOPE_2_3":
		case "EQ_TARGET_SCOPE_2_4":
		case "EQ_TARGET_SCOPE_2_5":
		case "EQ_TARGET_SCOPE_3_3":
		case "EQ_TARGET_SCOPE_3_4":
		case "EQ_TARGET_SCOPE_3_5":
		case "EQ_TARGET_SCOPE_4_3":
		case "EQ_TARGET_SCOPE_4_4":
		case "EQ_TARGET_SCOPE_4_5":
		case "EQ_TARGET_SCOPE_5_3":
		case "EQ_TARGET_SCOPE_5_4":
		case "EQ_TARGET_SCOPE_5_5":
		case "EQ_TARGET_SCOPE_6_3":
		case "EQ_TARGET_SCOPE_6_4":
		case "EQ_TARGET_SCOPE_6_5":
		case "EQ_TARGET_SCOPE_0_7":
		case "EQ_TARGET_SCOPE_1_7":
		case "EQ_TARGET_SCOPE_2_7":
		case "EQ_TARGET_SCOPE_3_7":
		case "EQ_TARGET_SCOPE_4_7":
		case "EQ_TARGET_SCOPE_5_7":
		case "EQ_TARGET_SCOPE_6_7":
		case "EQ_TARGET_SCOPE_0_8":
		case "EQ_TARGET_SCOPE_1_8":
		case "EQ_TARGET_SCOPE_2_8":
		case "EQ_TARGET_SCOPE_3_8":
		case "EQ_TARGET_SCOPE_4_8":
		case "EQ_TARGET_SCOPE_5_8":
		case "EQ_TARGET_SCOPE_6_8":
		case "EQ_TARGET_SCOPE_0_9":
		case "EQ_TARGET_SCOPE_1_9":
		case "EQ_TARGET_SCOPE_2_9":
		case "EQ_TARGET_SCOPE_3_9":
		case "EQ_TARGET_SCOPE_4_9":
		case "EQ_TARGET_SCOPE_5_9":
		case "EQ_TARGET_SCOPE_6_9":
		case "EQ_TARGET_SCOPE_CONFIG_CAMERA":
		case "EQ_TARGET_SCOPE_CONFIG_CENTRE":
		case "EQ_TARGET_SCOPE_CONFIG_CENTRE_COLOUR":
		case "EQ_TARGET_SCOPE_CONFIG_CONTRAST":
		case "EQ_TARGET_SCOPE_CONFIG_GLOW":
		case "EQ_TARGET_SCOPE_CONFIG_MENU_OFF":
		case "EQ_TARGET_SCOPE_CONFIG_MENU_ON":
		case "EQ_TARGET_SCOPE_CONFIG_MODE":
		case "EQ_TARGET_SCOPE_CONFIG_OPACITY":
		case "EQ_TARGET_SCOPE_CONFIG_RETICLE":
		case "EQ_TARGET_SCOPE_CONFIG_SENSITIVITY":
		case "EQ_TARGET_SCOPE_CONFIG_SIZE":
			{
			player.ship.setEquipmentStatus(equipment,"EQUIPMENT_OK");
			if (this.version < 2) log(this.name, equipment + " repaired automatically.");
			return;
			}
 		default:
			{
			if (this.version < 1) log(this.name, equipment + " not repaired.");
			return;
			}
		}
	}

/* ====================================================================================
			THE END
======================================================================================= */
